#ifndef TDC7200_H
#define TDC7200_H

// parts of this code taken from https://github.com/TAPR/TICC/tree/master/TICC

#define SPIfrequency (uint32_t) 2500000
#define EXTCLOCK (uint32_t) 16E6 //Value of external CLOCK for 
#define AVG_CYCLES 10


// TDC7200 register addresses
const int CONFIG1 =		0x00;           // default 0x00
const int CONFIG2 =		0x01;           // default 0x40
const int INT_STATUS =		0x02;           // default 0x00
const int INT_MASK =		0x03;           // default 0x07
const int COARSE_CNTR_OVF_H =	0x04;       	// default 0xff
const int COARSE_CNTR_OVF_L =	0x05;       	// default 0xff
const int CLOCK_CNTR_OVF_H =	0x06;        	// default 0xff
const int CLOCK_CNTR_OVF_L =	0x07;        	// default 0xff
const int CLOCK_CNTR_STOP_MASK_H = 0x08;	// default 0x00
const int CLOCK_CNTR_STOP_MASK_L = 0x09;	// default 0x00
// gap from 0x0A thru 0x0F...
const int TIME1 =		0x10;           // default 0x00_0000
const int CLOCK_COUNT1 =	0x11;           // default 0x00_0000
const int TIME2	=		0x12;           // default 0x00_0000
const int CLOCK_COUNT2 =	0x13;           // default 0x00_0000
const int TIME3	=		0x14;           // default 0x00_0000
const int CLOCK_COUNT3 =	0x15;           // default 0x00_0000
const int TIME4	=		0x16;           // default 0x00_0000
const int CLOCK_COUNT4 =	0x17;           // default 0x00_0000
const int TIME5	=		0x18;           // default 0x00_0000
const int CLOCK_COUNT5 =	0x19;           // default 0x00_0000
const int TIME6	=		0x1A;           // default 0x00_0000
const int CALIBRATION1 =	0x1B;           // default 0x00_0000
const int CALIBRATION2 =	0x1C;           // default 0x00_0000



// Chip properties:
// Most Significant Bit is clocked first (MSBFIRST)
// clock is low when idle
// data is clocked on the rising edge of the clock (seems to be SPI_MODE0)
// max clock speed: 20 mHz

class tdc7200 {
  private:
    float clockperiod = 1.0 / EXTCLOCK;
  
  public:
    uint32_t time1_rx = 0; 
    uint32_t time2_rx = 0;
    uint32_t time3_rx = 0;
    uint32_t time4_rx = 0;
    uint32_t time5_rx = 0;
    uint32_t time6_rx = 0;
    
    uint32_t clockcount1_rx = 0;
    uint32_t clockcount2_rx = 0;
    uint32_t clockcount3_rx = 0;
    uint32_t clockcount4_rx = 0;
    uint32_t clockcount5_rx = 0;
    
    uint32_t cal1_rx = 0;
    uint32_t cal2_rx = 0;

// variables for tx
    uint32_t time1_tx = 0; 
    uint32_t time2_tx = 0;
    uint32_t time3_tx = 0;
    uint32_t time4_tx = 0;
    uint32_t time5_tx = 0;
    uint32_t time6_tx = 0;
    
    uint32_t clockcount1_tx = 0;
    uint32_t clockcount2_tx = 0;
    uint32_t clockcount3_tx = 0;
    uint32_t clockcount4_tx = 0;
    uint32_t clockcount5_tx = 0;
    
    uint32_t cal1_tx = 0;
    uint32_t cal2_tx = 0;

    float tof1 = 0;
    float tof2 = 0;
    float tof3 = 0;
    float tof4 = 0;
    float tof5 = 0;
    
    //READ 8bit Register of RX TDC7200
    byte readReg8(byte address) {
      byte inByte = 0;
    
      SPI.beginTransaction(SPISettings(SPIfrequency, MSBFIRST, SPI_MODE0));
      // take the chip select low to select the device:
      digitalWrite(15, LOW);
    
      SPI.transfer(address & 0x1f);
      inByte = SPI.transfer(address);
    
      digitalWrite(15, HIGH);
      SPI.endTransaction();
    
      return inByte;
    }
 // TX TDC7200   
 byte readReg8_TX(byte address) {
      byte inByte_tx = 0;
    
      SPI.beginTransaction(SPISettings(SPIfrequency, MSBFIRST, SPI_MODE0));
      // take the chip select low to select the device:
      digitalWrite(14, LOW);
    
      SPI.transfer(address & 0x1f);
      inByte_tx = SPI.transfer(address);
    
      digitalWrite(14, HIGH);
      SPI.endTransaction();
    
      return inByte_tx;
    }
    
    //READ 24bit Register
    uint32_t readReg24(byte address) {
      uint32_t long value = 0;
    
      // CSB needs to be toggled between 24-bit register reads
      SPI.beginTransaction(SPISettings(SPIfrequency, MSBFIRST, SPI_MODE0));
        digitalWrite(15, LOW);
      
        SPI.transfer(address & 0x1f);
      
        uint16_t msb = SPI.transfer(address);
        uint16_t mid = SPI.transfer(address);
        uint16_t lsb = SPI.transfer(address);
      
        value = (msb << 16) + (mid << 8) + lsb;
      
        digitalWrite(15, HIGH);
      SPI.endTransaction();
      delayMicroseconds(5);
      return value;
      
    }

    //READ 24bit Register of TX TDC7200
    uint32_t readReg24_TX(byte address) {
      uint32_t long value_tx = 0;
    
      // CSB needs to be toggled between 24-bit register reads
      SPI.beginTransaction(SPISettings(SPIfrequency, MSBFIRST, SPI_MODE0));
        digitalWrite(14, LOW);
      
        SPI.transfer(address & 0x1f);
      
        uint16_t msb = SPI.transfer(address);
        uint16_t mid = SPI.transfer(address);
        uint16_t lsb = SPI.transfer(address);
      
        value_tx = (msb << 16) + (mid << 8) + lsb;
      
        digitalWrite(14, HIGH);
      SPI.endTransaction();
      delayMicroseconds(5);
      return value_tx;
    }

    //WRITE REGISTER OF RX TDC7200
    void writeReg(byte address, byte value) {
       uint16_t test = 0;  
      // take the chip select low to select the device:
      SPI.beginTransaction(SPISettings(SPIfrequency, MSBFIRST, SPI_MODE0));
        digitalWrite(15, LOW);
        // Force Address bit 6 to one for a write
        SPI.transfer16((address | 0x40) << 8 | value);
        test = (address | 0x40) << 8 | value;
        //Serial.print("-----test = "); Serial.println(test, BIN);
        digitalWrite(15, HIGH);
      SPI.endTransaction();
    } 

//WRITE REGISTER IF TX TDC7200
    void writeReg_TX(byte address, byte value) {
       uint16_t test_tx = 0;  
      // take the chip select low to select the device:
      SPI.beginTransaction(SPISettings(SPIfrequency, MSBFIRST, SPI_MODE0));
        digitalWrite(14, LOW);
        // Force Address bit 6 to one for a write
        SPI.transfer16((address | 0x40) << 8 | value);
        test_tx = (address | 0x40) << 8 | value;
        //Serial.print("-----test = "); Serial.println(test, BIN);
        digitalWrite(14, HIGH);
      SPI.endTransaction();
    } 

    //COMPUTE TOFs
    float getTOF_rx(uint32_t timeN, uint32_t timeN1, uint32_t clockcountN, uint32_t cal1, uint32_t cal2) {
      float calCount_rx = (cal2 - cal1) / (AVG_CYCLES - 1.0);
      //Serial.print("clockperiod = "); Serial.println(clockperiod,8);
      //Serial.print("calCount = "); Serial.println(calCount);
      float normLSB_rx = (clockperiod / calCount_rx);
      //Serial.print("normLSB = "); Serial.println(normLSB,16);
      //float fehler = timeN-timeN1
      //float tof_temp_rx = normLSB_rx * (float(timeN) - (float)timeN1) + clockcountN * clockperiod;
      float tof_temp_rx = normLSB_rx* float(timeN);
//      Serial.print("tof_temp = "); Serial.println(tof_temp, 16);

      
      return tof_temp_rx;
    }


        float getTOF_tx(uint32_t timeN, uint32_t timeN1, uint32_t clockcountN, uint32_t cal1, uint32_t cal2) {
      float calCount_tx = (cal2 - cal1) / (AVG_CYCLES - 1.0);
      //Serial.print("clockperiod = "); Serial.println(clockperiod,8);
      //Serial.print("calCount = "); Serial.println(calCount);
      float normLSB_tx = (clockperiod / calCount_tx);
      //Serial.print("normLSB = "); Serial.println(normLSB,16);
      //float fehler = timeN-timeN1
      //float tof_temp_tx = normLSB_tx * (float(timeN) - (float)timeN1) + clockcountN * clockperiod;
      float tof_temp_tx = normLSB_tx* float(timeN);
      
      return tof_temp_tx;
    }
    
    
};

#endif	/* TDC7200_H */
